@@ -977,7 +977,7 @@ def plotINDEL_same (matrix_path, matrix_path_clustered, matrix_path_nonClustered
 
 
 
-def rainfall (chrom_based_IMD, project, project_path, chrom_path, chromLengths, centromeres, contexts, includedVAFs, correction=True, windowSize = 10000000, bedRanges=None):
+def rainfall (chrom_based_IMD, project, project_path, chrom_path, chromLengths, centromeres, contexts, includedVAFs, includedCCFs, correction=True, windowSize = 10000000, bedRanges=None):
 	'''
 	Generates rainfall plots when subClassify is True.
 
@@ -1050,7 +1050,7 @@ def rainfall (chrom_based_IMD, project, project_path, chrom_path, chromLengths,
 
 
 	headerFields = ["project", "samples","ID","genome","mutType","chr","start","end", "ref", "alt", "mutClass", "IMDplot", "group", "IMD", "vaf","class", "failedReason"]
-	if not includedVAFs:
+	if not includedVAFs and not includedCCFs:
 		headerFields = ["project", "samples","ID","genome","mutType","chr","start","end", "ref", "alt", "mutClass", "group", "IMDplot","IMD", "class", "failedReason"]
 	# centromeres = {'GRCh38':{'1': [122026460,125184587],'10': [39686683,41593521],'11':[51078349,54425074],'12':[34769408,37185252],'13': [16000001,18051248],
 	# 			'14': [16000001,18173523],'15': [17000001,19725254],'16': [36311159,38280682],'17': [22813680,26885980],'18': [15460900,20861206],
@@ -1121,7 +1121,8 @@ def rainfall (chrom_based_IMD, project, project_path, chrom_path, chromLengths,
 			except:
 				continue
 
-	samplesSet = set([str(x) for x in list(samplesSet)])
+	samplesSet = set([str(x) for x in list(samplesSet) if x != "."])
+
 	# Set up chromosome parameters:
 	# chrom_path = "/anaconda3/lib/python3.6/site-packages/SigProfilerMatrixGenerator/references/chromosomes/tsb/" + genome + "/"
 	chroms = [x.split(".")[0] for x in os.listdir(chrom_path) if x != ".DS_Store" and x[0]!= "G" and x[0] != "M" and x != "BED_" + genome + "_proportions.txt" and 'proportions' not in x]
